/* adder.cc {\documentstyle{jsrc2latex}\rhead{\sl adder.cc\quad\thepage}
   \font\tenrm=cmtt10}饹 Adder μ¸
{\ \begin{verbatim}
$Revision: 1.1 $ $Date: 2000/12/02 10:10:05 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode

}% end of latex-mode }
*/


/* adder.cc
----------------------------------------------------------------
   饹 Adder μ¸
----------------------------------------------------------------
*/

#include <stdio.h>
#include <iostream>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "common.h"                                 /* ̴Ķ */
#include "list.h"                               /* 饹 List  */
#include "element.h"                         /* 饹 Element  */
#include "p_tmp.h"         /* ƥץ졼ȥ饹 Receive, Send  */
//#include "p_tmp.cc"
#include "adder.h"	    		      	       /* 饹 Adder  */


Boolean Adder::Init(char* o_name, int o_num, int numinput, char* p_str)
{
	/*
	 Adder::Init  ؿ
	*/

	ninput = numinput;

	if ((rp = new Receive<double>[ninput]) == NULL) {
		Eprint(__FILE__, __LINE__, "Cannot allocate memory for rp.");
		fprintf(stderr, "object '%s %d'"
			   " : Adder::Adder\n", object_name, object_number);
		exit(1);
	}

	for (int i = 0; i < ninput; i++) rp[i].Init("rp", i);

	return Element::Init(o_name, o_num, 0, p_str);
}

// for SCF-style model building
// added by iYutaka (Mon Apr 16 19:21:19 2001)
Boolean Adder::Init(char* o_name, int o_num, FILE* fp)
{
	int numinput;
	assert(fscanf(fp, "%d", &numinput) == 1);
	return Init(o_name, o_num, numinput);
}

void Adder::Register_receive_ports(void)
{
	/*
	 Adder::Register_receive_ports  üҤϿ
	*/

	for (int i = 0; i < ninput; i++) Register((void*)(&rp[i]));
}


void Adder::Set_initial_state(FILE* fp)
{
	/*
	 Adder::Set_initial_state  ̤
	*/

	time_t tnow;
    if (fscanf(fp, "%ld", &tnow) != 1) {
		Eprint(__FILE__, __LINE__, 
			"Cannot scan infomation for initializing.");
		fprintf(stderr, "object '%s %d'"
			   " : Adder::Set_initial_state\n",
			   object_name, object_number);
		exit(1);
	}
	if (tnow > 0) current_time = tnow;
}

void Adder::Set_initial_state(time_t ct)
{
	/*
	 Adder::Set_initial_state  ̤
	*/
	if (ct > 0) current_time = ct;
}

void Adder::Save_terminal_state(FILE* fp)
{
	/*
	 Adder::Save_terminal_state  ǽ̤¸
	*/
	
	if (fp != NULL) {
    	fprintf(fp, "%ld", current_time);
    }
}

void Adder::Save_terminal_state(time_t& ct)
{
	/*
	 Adder::Save_terminal_state  ǽ̤¸
	*/
	ct = current_time;
}

time_t Adder::Calculate_time_step(void)
{
	/*
	 Adder::Calculate_time_step  ׻ֳִ֤η׻
	*/

	time_t dt, min_of_time_forward = 1000000;
	void *dp_l;
	
	for (int i = 0; i < ninput; i++) {
		((Port*) &rp[i])->Get_data_pack(current_time, dp_l);
		if ((dt = ((Data_pack*)dp_l)->Data_time()
			- current_time) == 0) {
			if (((Port*) &rp[i])->Get_data_pack(current_time + 1, dp_l)
				== YES) {
			    dt = ((Data_pack*)dp_l)->Data_time() - current_time;
			}
			else {
				return 0; // ʾʤʤ 
			}
		}
		min_of_time_forward
			= ((dt < min_of_time_forward) ? dt : min_of_time_forward);
	}

	return min_of_time_forward;
}

Boolean Adder::Initial_output(void)
{
	/*
	 Adder::Initial_output  ν
	*/

	sum_inflow = 0.;
	for (int i = 0; i < ninput; i++) {
		if (rp[i].Can_you_get_data(current_time, current_time) == NO) {
			return NO;
		}
		sum_inflow += rp[i].Get_data(current_time);
	}

	sp.Send_data(current_time, sum_inflow);
	// printf("Adder::Initial_output done\n");
	return YES;
}


Boolean Adder::Calculate(void)
{
	/*
	 Adder::Calculate  ήз׻
	*/

	sum_inflow = 0.;
	for (int i = 0; i < ninput; i++) 
		sum_inflow += rp[i].Get_data(current_time + time_step);

	/* ǡν */
	sp.Send_data(current_time + time_step, sum_inflow);
	return YES;
}

Boolean Adder::Work(void)
{
	if (current_time >= Terminal_time()) return NO;		// ׻Ѥ. 
	Boolean calc = NO;

  LOOP:
	if ((time_step = Calculate_time_step()) == 0) goto FINISH;
	if (Calculate()==NO) goto FINISH;  
	calc = YES;
	if ((current_time += time_step) >= Terminal_time()) goto FINISH;
	goto LOOP;

  FINISH:
	if (calc == YES) {
		Set_data_pack_array();		// ǡѥåκ. 
		Reduce_data_pack_stock();	// ץǡѥåξõ. 
	}
	return calc;
}
 
/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $
Alterations from Revision: 1.14
Initial_output was updated so that now it returns a value of type
Boolean. 

Revision: 1.11 ѹ
Work ޤ

Revision: 1.6 ѹ
hymoshist.doc 110 ȼѹ

Revision: 1.3 ѹ
sum_inflow ֤Ȥɤ߹ɬפʤʤޤ

Revision: 1.2 ѹ
Хν

Revision: 1.1 ѹ
¤Ǥ뤫Υåߤޤ

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */

