/* delay.cc {\documentstyle{jsrc2latex}\rhead{\sl delay.cc\quad\thepage}
   \font\tenrm=cmtt10}
{\ \begin{verbatim}
$RCSfile: delay.cc,v $  $Revision: 1.1 $ 
$Date: 2000/12/02 10:04:51 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode
}% end of latex-mode }

Implementation of class Delay<T>
*/

#include <stdio.h>
#include <time.h>
#include <assert.h>
#include <stdlib.h>
#include "p_tmp.h"
#include "delay.h"

#include "ostring.h"
#include "vary.h"

template <class T>
void Delay<T>::Set_parameter(FILE* fp) 
{
    const int BUFFLEN = 256;
    char buffer[BUFFLEN];

    Get_line(fp, buffer, BUFFLEN, __FILE__, __LINE__);
    if (sscanf(buffer, "%ld", &timeLag) != 1) {
        Eprint(__FILE__, __LINE__); Oprint(); exit(1);
    }
	assert(timeLag >= 0);
}

// 10
template <class T>
void Delay<T>::Set_initial_state(FILE* fp)
{
    const int BUFFLEN = 256;
    char buffer[BUFFLEN];
     char* dstr = new char[BUFFLEN];
    time_t cp;
    Get_line(fp, buffer, BUFFLEN, __FILE__, __LINE__);
    // It is assumed that current_time and value T are written in
    // one line 

    str2dstr(buffer, dstr, BUFFLEN);
    dstr2data(dstr, cp);
    dstr2data(dstr, newestValue);
    if (cp > 0) current_time = cp;
}

// 16
template <class T>
Boolean Delay<T>::Initial_output(void)
{
    if (rp.Number_of_data_packs() == 0) return NO;
	sp.Send_data(current_time, newestValue);
	return YES;
}

template <class T>
time_t Delay<T>::Calculate_time_step(void)
{
	time_t t;  void* dp_l;

	t = current_time - timeLag;
	if (((Port*) &rp)->Get_data_pack(t + 1, dp_l) == NO) return 0;
	return (((Data_pack*)dp_l)->Data_time() - t);
}

// 19
template <class T>
Boolean Delay<T>::Calculate(void) 
{
	void* dp;
	time_t ta = current_time + time_step - timeLag;
	if (((Port*) &rp)->Get_data_pack(ta, dp) == NO) return NO;
	newestValue = ((Data<T>*)dp)->Value();
	sp.Send_data(ta, newestValue);
	return YES;
}

// 20
template <class T>
Boolean Delay<T>::Work(void)
{
	if (current_time >= Terminal_time()) return NO;		
	// We do notneed to work.

	Boolean calc = NO;

  LOOP:
	if ((time_step = Calculate_time_step()) == 0) goto FINISH;
	if (Calculate()==NO) goto FINISH;  
	calc = YES;
	if ((current_time += time_step) >= Terminal_time()) goto FINISH;
	goto LOOP;

  FINISH:
	if (calc == YES) {
		Set_data_pack_array();		// Make data packs for sending
		Reduce_data_pack_stock();	// Delete unnecessary data packs
	}
	return calc;
}

// 21
template <class T>
time_t Delay<T>::Necessary_time_from(Receive_port*)
{
	return (current_time - timeLag);
}

template class Delay<double>;
template class Delay<float>;
template class Delay<int>;
template class Delay<long>;
template class Delay<OHyMosString>;

// for Vary<T>

typedef Vary<int> Viary;
typedef Vary<long> Vlary;
typedef Vary<float> Vfary;
typedef Vary<double> Vdary;

template class Delay<Viary>;
template class Delay<Vlary>;
template class Delay<Vfary>;
template class Delay<Vdary>;

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $
Change in Revision 1.3
Bug fix

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */

