/*;  scfsub.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision$$ {\rhead{\sl scfsub.h\quad\thepage}} */
/*;  $$Date$$  */
/*;  $$Source$$  */
/*;  $$RCShost$$  */

// @(#)  scfsub.h -- subsystem created from a structure configuration file

/*
{\ {\setlength{\parskip}{4pt}% latex-mode
...
}% end of latex-mode }
*/

#if !defined(scfsub_h)
#define scfsub_h

#include "sub.h"

/*----------------------------------------------------------------------
CLASS
    ScfSubSystem

    Sub system class confugured from a "structure configuration file"

DESCRIPTION
    subsytem class configured from a structure configuration file

    <japanese>
    ScfSubSystem ϡüҤ乽Ǥιե Structure
    Configuration File ɤߤȤäƺꤢʬϤΥ饹Ǥ

    </japanese>

----------------------------------------------------------------------*/

class ScfSubSystem : public Sub_system {
protected:
  //////////////////////////////////////////////////////////////////////
  // Pointer to the area where pointers to relay ports for receiveing
  // data are stored
  Port** mRps;
  //////////////////////////////////////////////////////////////////////
  // Number of relay ports for receiving data
  int mNumRps;

  //////////////////////////////////////////////////////////////////////
  // Pointer to the area where pointers to relay ports for sending
  // data are stored
  Port** mSps;
  //////////////////////////////////////////////////////////////////////
  // Number of relay ports for sending data
  int mNumSps;

  //////////////////////////////////////////////////////////////////////
  // Usual (Non-iterate) Components
  Element** mElmnt;

  //////////////////////////////////////////////////////////////////////
  // Number of usual components
  int mNumElmnt;

  //////////////////////////////////////////////////////////////////////
  // Iterator Components
  Element** mIte;

  //////////////////////////////////////////////////////////////////////
  // Number of iterator components
  int mNumIte;

  //////////////////////////////////////////////////////////////////////
  // Structure Configuration File
  char* mScffile;

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ¦üҤϿޤ饹ޤConstruct 
  // ƤФޤϿƤʤ¦üҤ³Ǥޤ.
  // </japanese>
  void Register_receive_ports(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ¦üҤϿޤ饹ޤConstruct 
  // ƤФޤϿƤʤ¦üҤ³Ǥޤ.
  // </japanese>
  void Register_send_ports(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ǤΤ, ȿ׻Ԥʤʤ֥ȤΥɥ쥹
  // component_array Ͽޤ.
  //
  // ϿƤʤ֥Ȥˤ, ׻¹̿᤬ޤ.
  //
  // 㤨, 饹ǹ֥ abc ǡФȤ
  // , Τ褦 abc Ͽޤ.
  // <pre>
  //   void Register_components(void) { Register((void*) &abc); }
  // </pre>
  // λ, ɬ &abc  void* ˥㥹ȤƲ.
  //
  // ˤ &abc , Ū object_list ˼Ͽ줿, ǽŪ
  // void*  component_array ˵ޤ.
  // (Sub_system::Construct )
  //
  // </japanese>
  virtual void Register_components(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Register_iterators , ǤΤ, ȿ׻ԤʤǤΥ
  // 쥹 iterator_array Ͽޤ. ȿ׻Ԥʤʬϥǥ
  // ˺ץޤδؿʤФʤޤ
  //
  // ScfSubSytem Ѥ Register_iterators ϡscfsub.cc ǼƤޤ
  // </japanese>
  void Register_iterators(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ʬϥǥüҤ³ Sub_system::Connect Ѥ
  // ޤ.
  //
  // ScfSubSytem Ѥ Connect_ports scfsub.cc ǼƤޤ
  // </japanese>
  void Connect_ports(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ʬϥǥľ̿³ Element::Register(void*,int) 
  // Ѥޤ.
  //
  // ScfSubSytem Ѥ Connect_components scfsub.cc ǼƤޤ
  // </japanese>
  void Connect_components(void);

public:
  //////////////////////////////////////////////////////////////////////
  // Returns a pointer to a new ScfSubSystem
  ScfSubSystem* NewElement() { return new ScfSubSystem(); }

  //////////////////////////////////////////////////////////////////////
  // Initialize the element with the information obtained from the file
  // pointer
  //
  // [in] objectname -- object name
  // [in] objnum     -- object number
  // [in] fp         -- file pointer
  Boolean Init(char* objectname, int objnum, FILE* fp);

  //////////////////////////////////////////////////////////////////////
  // Default Constructor
  ScfSubSystem()
    : Sub_system(), mRps(NULL), mSps(NULL), mElmnt(NULL), mIte(NULL),
      mScffile(NULL)
  { class_name = "ScfSubSystem"; }

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~ScfSubSystem() { delete [] mRps; delete [] mSps; delete [] mElmnt;
  delete [] mIte; delete [] mScffile; }

};

#endif // scfsub_h

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
