/*;  input.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl input.h\quad\thepage}} */
/*;  $$Date: 2000/12/02 10:00:56 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/input.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#)  input.h

#if !defined(input_h)
#define input_h

#include <stdio.h>

#include "send.h"       //; 饹 Send_port 

/*----------------------------------------------------------------------
CLASS

    Input_port

    abstract class for input ports

DESCRIPTION
    Abstract class for creating input ports for a total system.

    This is the base class from which the user creates his own input
    port models.

    Do not confuse Input_port with Receive_port.

    <japanese>
    ܷüҤϡηϤüҤ뤿ݥ饹Ǥ桼
    ϡΥ饹ѾơηϤüҤΥǥƤ
    ܷüҤΥ饹Ǥϡ桼¸٤üҤδ
    ܻͤƤޤޤμǤϡѰդǤ뵡ǽ
    Ƥޤ

    ηϥǥϡδܷüҤΥ饹Ѿƥ桼
    üҤ̤ƥǡꤹΤȤ߷פƤޤ

    ηϤФüҤϥǡȤ򤹤Ȥ
    ̣ǤϡüҤϡܷǥǥμüҤ˻Ƥޤ
    ΤΤȹͤ褤Ǥ

    ηϥǥ, ե, ͥåȥˤ, ܡ
    , CRT Ϥʤ, ¿ʥǡϤбʤФʤޤ
    ηϥǥ, ¿ʥǡϤ˼, 
     Data_pack ΥǡѴʤФʤޤ󡣤ޤߥ졼
    ̤򥷥ߥ졼¹ԼԤ󶡤, Data_pack 
    Ƥǡ򥷥ߥ졼¹ԼԤ˾ǡѴʤ
    Фʤޤ󡣤Τ褦ʥǡѴ˴ؤŪ, 桼
    ü, üҤǹԤȤˤ, ηϥǥǤ, 
    üҡüҤμ³뤳Ȥˤޤʤ
    üҡüҤϡ귿ŪʽȳνȤΩ򤹤뤳
    ˤʤޤ

    </japanese>

----------------------------------------------------------------------*/

//;  饹 Input_port .

class Input_port : public Send_port {
  //; ܷüҤΥǥ

//;  1.  ³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the string which shows data-type treated by the current
  // input-port.  Because this is pure virtual, the function must be
  // defined in derived classes from this class.
  //
  // <dl> <dt>Example
  // <dd> char* Data_pack_type(void) { return "INT"; }
  // </dl>
  //
  // We call the string returned by this funcion "data_pack_type".
  // <japanese>
  // ϥǡѥå֤
  //
  // 貾۴ؿǤ. Input_port 饹, ϥǡѥå
  // ɽʸؤΥݥ󥿤֤褦ɬƲ.
  //
  // 㤨, ϥǡѥå "INT" Ǥ饹Ǥ,
  // <pre>
  // char* Data_pack_type(void) { return "INT"; }
  // </pre>
  // Τ褦ޤ.
  //
  // ³ˤ, üҤüҤȤ Data_pack_type פ뤳
  // Ȥǧޤ.
  //
  // <pre>
  // RETURN:
  //   char*     ϥǡѥåɽʸؤΥݥ.
  // </pre>
  // </japanese>
  virtual char* Data_pack_type(void) = 0;

//;  2.  ϥǡǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ϥǡ
  //
  // üҤ, ϥǡǡѥå, ǡѥå
  // ؿƲ.
  // </japanese>
  virtual void Input_data(void) = 0;

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // եϡ׻ǽʤΤǡ׻äƤɤ礦ʤ
  // </japanese>
  void Creq(void) {
    Eprint(__FILE__, __LINE__); exit(1);
  }

//;  3.  ɽǽ.
    //; Input_port ǤϺޤ.

//;  4.  ǽ, 󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  // Execute Send_port::Init and returns a Boolean value which is
  // returned by Send_port::Init
  //
  // <japanese>
  // Ѵؿ
  //
  // Send_port::Init ¹Ԥޤ.
  // <pre>
  //    RETURN:
  //        YES   .
  //        NO    Ť˽줿. (Object::Init)
  //    EXIT:
  //        5     object_name εΰ褬ݤǤʤ.
  //                (Object::Init)
  // </pre>
  // </japanese>
  Boolean Init(char* p_name, int p_num)
  { return Send_port::Init(p_name, p_num); }

  //////////////////////////////////////////////////////////////////////
  // Constructor.  Constructs by the use of default constructor of base
  // class Send_port and initializes the data members by the use of
  // Input_port::Init.
  //
  // <japanese>
  // Input_port(char*,int) , 쥯饹 Send_port Υǥեȥ󥹥
  // 饯Ѥ, Input_port::Init Ѥ, Фʲ̤
  // ޤ.
  //
  // <pre>
  //        class_name            : "Input_port"
  //        object_name           : p_name (ҡפ˥ԡ)
  //        object_number         : p_num
  //        owner                 : NULL
  //        buffer_list           : ҡפ˳ݤ. (³)
  //        receiver_array        : NULL
  //        number_of_receivers   : -1
  //        data_pack_array       : NULL
  //        number_of_data_packs  : -1
  // </pre>
  // </japanese>
  Input_port(
    char* p_name,       // object_name
    int p_num           // object_number
    );

  //////////////////////////////////////////////////////////////////////
  // Constructor
  //
  // <japanese>
  //
  // ǥեȥ󥹥ȥ饯, Input_port::Init ¹Ԥޤ. η
  // , ʲȤʤޤ.
  // <pre>
  //        object_name           : NULL
  //        object_number         : -1
  // </pre>
  //
  // ¾ΥФФ, Ĥ󥹥ȥ饯ƱǤ.
  // ξ, Input_port::Init ѤƥФƲ.
  // </japanese>
  Input_port(void);

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~Input_port(void);

};


#endif  // ! input_h

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $
\end{verbatim}\end{comment}
} */

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
