/*;  tram_plaw_storage.h {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl tram_storage_slope.h\quad\thepage}} */
/*;  $$Date: 2000/08/31 09:27:14 $$  */
/*;  $$Source: /home/kanazawa/kenkyu/models/sample/sample6/tram_storage_slope.h,v $$  */
/*;  $$RCShost: kizugawa.kuciv.kyoto-u.ac.jp $$  */

// @(#) tram_storage_slope
#if !defined(tram_storage_river_h)
#define tram_storage_river_h

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fstream.h>
#include <math.h>

#include "common.h"             //; ̴Ķ
#include "element.h"
#include "p_tmp.h"
#include "khmkdir.h"

/*----------------------------------------------------------------------
CLASS
  TramPowerLawStorage

  Power-law storage element model with multi inputs andd multi outputs

DESCRIPTION

  <japanese>
  [饹̾]TramPowerLawStorage

  [ǥ̾] 沴ؿαǥ

  [ǽ] 沴ؿαǥ

  ü
  <pre>
      O_i = \alhpa_i s^{m_i}
      ds/dt = a_1*I_1 + a_2*I_2 + ... + a_m*I_m - O_1 - O_2 - ... - O_n
  </pre>
   TRAM ˡˤäƷ׻ޤs (m^3) ϡαǤα̡
  a_1*I_1, a_2*I_2, ..., a_m*I_m ή (m^3/sec)O_1, O_2, ...,
  O_n ϡήж (m^3/sec) Ǥήж O_i ϡα s 沴ؿ
  \alhpa_i s^{m_i}ɽΤȤޤ

  [üҤλ]

  m ĤμüҤή I_1, I_2, ... I_mޤ

  <ul>
  <li> üҤθĿ m: Ǥա
  <li> üҤꤢĤǡѥåη: Data&lt;double&gt
  <li> üҤꤢĤǡñ: m^3/sec
  <li> üҤΥǡ³: ѥ᥿ǻ
  <li> üҤΥǡΨ a_i: ѥ᥿ǻꡢ̵
  <li> üҤΥ֥̾: rp
  <li> üҤΥֹ֥: 0, 1, ..., m-1
  </ul>

  a_i*I_i μ m^3/sec ˤʤ褦դСüҤ
  I_i Ȥμϡm^3/sec ǤʤƤ⹽ޤ

  [üҤλ]

  n ĤμüҤή O_1, O_2, ... O_mФޤ

  <ul>
  <li> üҤθĿ n: Ǥա
  <li> üҤꤢĤǡѥåη: Data&lt;double&gt
  <li> üҤꤢĤǡñ: m^3/sec
  <li> üҤΥǡ³: ǡ
  <li> üҤΥ֥̾: sp
  <li> üҤΥֹ֥: 0, 1, ..., n-1
  </ul>

  [Init Τɲþ]

  Init ΤɲþǤϡĤͤ¤٤ޤ
<pre>
  mNumRp, mNumSp, (Dct[i], mCqup[i], i=0, ..., mNumRp-1),
  (mAlpha[i], mM[i], i=0, ..., mNumSp-1), mSlevel, mEps,
  mFlag
</pre>
  ѿΰ̣ϤĤΤ褦Ǥ
  <ul>
  <li> int mNumRp ϼüҤθĿ
  <li> int mNumSp üҤθĿ
  <li> (int Dct[i], double mCqup[i], i=0, ..., mNumRp-1) ϡüҤ
       ǡ³Ψ
  <li> (double mAlpha[i], double mM[i], i=0, ..., mNumSp-1) ϡή
       O_i α s ȤδطηȻؿ
  <li> double mSlevel, double mEps ϡ̤ɸ (m^3) ȡͥ
       ٥ (㤨С0.001)
  <li> int mFlag ϡ˥ե˽Ϥ (1)  ݤ (0)
  </ul>

  Init Τɲþǻꤹǡϡǡ2*m + 2*n + 5 ĤǤ

  [ѥ᥿]

  ѥ᥿ϡĤ 2*m + 2*n + 3 ĤǤ

<pre>
  (Dct[i], mCqup[i], i=0, ..., mNumRp-1),
  (mAlpha[i], mM[i], i=0, ..., mNumSp-1), mSlevel, mEps, mFlag
</pre>

  <ul>
  <li> (int Dct[i], double mCqup[i], i=0, ..., mNumRp-1) ϡüҤ
       ǡ³Ψ(-)
  <li> (double mAlpha[i], double mM[i], i=0, ..., mNumSp-1) ϡή
       O_i α s ȤδطηȻؿ(m-sec, -)
  <li> double mSlevel, double mEps ϡ̤ɸ (m^3) ȡͥ
       ٥ (㤨С0.001)
  <li> int mFlag ϡ˥ե˽Ϥ (1)  ݤ (0)
  </ul>

  []

  <ol>
  <li> time_t current_time: ߻ (unix time)
  <li> double mS_crnt:  (m^3)
  <li> int mHalvedp: η׻ (1) ³׻ (0)  (-)
  <li> mrPsum_last: 쥹ƥåʬ (m^3)
  <li> time_t mDt_next: ׻ֹߤθ (sec)
  <li> int mTime: ߤη׻ֳִ֤Ǥڤɾ򥯥ꥢϢ³
        (-)
  </ol>

  ­η׻ (³η׻ǤϤʤȤ) ǤϡmHalvedp = YES,
  mTime = 0, mDt_next = 1 (sec), mrPsum_last = 0 ȤƤ
  </japanese>

----------------------------------------------------------------------*/

class TramPowerLawStorage : public Element {

  int mNumRp;                 //; ή receive port ο
  int mNumSp;                 //; ή send port ο
                              //; ؤΥݥ

  Boolean mFlag;              //; monitor ѤΥե饰. monitor Ϥ
                              //;  YES

  double* mAlpha;             //; ̤η(m^(1-mM)sec)
  int* mM;                    //; ̤λؿ(-)
  double* mCqup;              //; ήüή̤Ψ
  double mSlevel;             //; ̤δ (m^3)
  double mEps;                //; (㤨0.001)

  double mS_crnt;             //; ̤θ (m^3)

  double mrPsum_last;         //; ή̤ (m^3)
  double mSsmall;             //; ̤κʬ׻Ǥڤ
                              //; ٥ (m^3)
  double mS_next;             //; ̤μ (m^3)
  double mS_last;             //; ̤ (m^3)
  double mQ_crnt;             //; ή̤θ (m^3/sec)
  double mQ_next;             //; ή̤μ (m^3/sec)
  double mQ_last;             //; ή̤ (m^3/sec)
  Boolean mHalvedp;           //; ׻ϻ⤷Ϸ׻ֳִ֤
                              //; Ⱦʬˤľ夫:YES,Ǥʤ:
                              //; NO
  Boolean mDoubledp;          //; ׻ֳִ֤ܤˤʤä:YES,
                              //; Ǥʤ:NO
  int mTime;                  //; ߤη׻ֳִ֤Ǥڤ
                              //; ɾ򥯥ꥢϢ³
  int mTime_max;              //; mTime ξ mTime_max > mTime
                              //; Ȥʤä׻ֳִ֤ܤˤ
                              //; ͤϡ󥹥ȥ饯Ϳ롣

  long mDt_next;              //; η׻η׻ֳִ (sec)

protected:
  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ü
  //
  // ή(m^3/sec)ޤǡǽ
  // </japanese>
  Send<double>* mpSp;

  ////////////////////////////////////////////////////////////////
  //; üҤϿ
  void Register_send_ports(void);

  ////////////////////////////////////////////////////////////////
  //; ü
  Receive<double>* mpRp;

  //////////////////////////////////////////////////////////////////////
  //; üҤϿ
  void Register_receive_ports(void);

  ////////////////////////////////////////////////////////////////
  ofstream monitor_storage, monitor_deltaT;

public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ѥ᥿
  //
  // ǡɤ߼ϤĤΤ褦Ǥ
  // <pre>
  // for(int i = 0; i < mNumRp; i++) {
  //   int dct;
  //   fscanf(fp, "%d %lf", &dct, &mCqup[i]);
  //   mpRp[i].Set_data_connection_type(dct);
  // }
  // for (int i = 0; i < mNumSp; i++) {
  //   fscanf(fp, "%lf %d", &mAlpha[i], &mM[i]);
  // }
  // fscanf(fp, "%lf %lf", &mSlevel, &mEps);
  // fscanf(fp, "%d", &mFlag);
  // </pre>
  // </japanese>
  void Set_parameter(FILE* fp);

  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ֤ fp ɤ߼ޤ
  //
  // <pre>
  // fscanf(fp, "%ld %lf %lf %ld", &current_time, &mS_crnt,
  //   &mrPsum_last, &mDt_next);
  // </pre>
  // </japanese>
  void Set_initial_state(FILE* fp);

  ////////////////////////////////////////////////////////////////
  Boolean Initial_output(void);

  ////////////////////////////////////////////////////////////////
  time_t Calculate_time_step(void) { return mDt_next; }  //; Work0ξ
  //; ׻ϻ time step ޤǤΥǡ YES
  ////////////////////////////////////////////////////////////////
  Boolean Can_you_calculate(void) { return Can_you_calculate0();}
  ////////////////////////////////////////////////////////////////
  Boolean Calculate(void);
  ////////////////////////////////////////////////////////////////
  Boolean Work(void);
  ////////////////////////////////////////////////////////////////
  void Save_terminal_state(FILE* fp);

  // Returns a pointer to a new TramStorageRiver
  TramPowerLawStorage* NewElement(void);

public:
  //; 󥹥ȥ饯
  TramPowerLawStorage(char* o_name, int o_num);
  TramPowerLawStorage(void);
  //; ǥȥ饯
  ~TramPowerLawStorage(void);

  // ¤ե init ؿ"Init Τɲþ" ʬˤϡ
  // üҤθĿ񤭤ޤ
  Boolean Init(char* o_name, int o_num, FILE* fp);

  Boolean Init(char* o_name, int o_num);

};// end of class TramPowerLawStorage

#endif

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
