/*;  tram_storage_slope.h {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl tram_storage_slope.h\quad\thepage}} */
/*;  $$Date: 2000/09/19 15:53:02 $$  */
/*;  $$Source: /home/kanazawa/kenkyu/models/ohparts/tram_storage_slope.cc,v $$  */
/*;  $$RCShost: kizugawa.kuciv.kyoto-u.ac.jp $$  */

// @(#) tram_storage_slope
#include "tram_storage_slope.h"

////////////////////////////////////////////////////////////
//; class TramStorageSlope ˴ؤ뤪á                 //
////////////////////////////////////////////////////////////

// const long MaxDT = 16384;  //; Ȥꤢ 214
const long MaxDT = 2048;  //; Ȥꤢ 211

//;  󥹥ȥ饯ؿ
TramStorageSlope::TramStorageSlope(char* o_name, int o_num)
  : Element(),
    mCqup(NULL), mSpSlope("spdis", 0), mpRpSlope(NULL), mRpClm("rpclm", 0)
{
  class_name = "TramStorageSlope";
  Init(o_name, o_num);
}

Boolean
TramStorageSlope::Init(char* o_name, int o_num)
{
  return Element::Init(o_name, o_num, 0, NULL);
}

TramStorageSlope::TramStorageSlope(void)
  : Element(),
    mCqup(NULL), mSpSlope("spdis", 0), mpRpSlope(NULL), mRpClm("rpclm", 0)
{
  class_name = "TramStorageSlope";
}

TramStorageSlope::~TramStorageSlope(void)
{
  delete [] mpRpSlope;
  delete [] mCqup;
}

Boolean
TramStorageSlope::Init(char* oname, int onum, FILE* fp)
{
  Object::Init(oname, onum);

  // read the number of receive ports from fp
  if (fscanf(fp, "%d", &mNumRps) != 1) {
    Eprint(__FILE__, __LINE__); exit(1);
  }

  if (mNumRps > 0) {
    mpRpSlope = new Receive<double>[mNumRps];
    NEWCHECK(mpRpSlope);
    mCqup = new double[mNumRps];
    NEWCHECK(mCqup);
  }

  for (int i = 0; i < mNumRps; i++) {
    mpRpSlope[i].Init("rpdis", i);
    mCqup[i] = 1.0;
  }
  Set_parameter(fp);

  return YES;
}

//;  ֥Ȥεΰ
TramStorageSlope*
TramStorageSlope::NewElement()
{
  return new TramStorageSlope();
}

//;  üҡüҤ
void
TramStorageSlope::Register_send_ports(void)
{
  Register((void*)&mSpSlope);
}

void
TramStorageSlope::Register_receive_ports(void)
{
  for (int i = 0; i < mNumRps; i++) {
    Register((void*)&(mpRpSlope[i]));
  }
  Register((void*)&mRpClm);
}

//;  ѥ᥿
void
TramStorageSlope::Set_parameter(FILE* fp)
{
  //; åοʿĹ̡ۡʿ
  //; Ѥɤ߹
  double wgrd;
  double ratio;
  if (fscanf(fp, "%ld %d %lf %lf %lf %lf %lf",
    &mNum, &mAtr, &mLength, &wgrd, &mArea, &ratio, &mUpperArea)
    != 7) {
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mNum <= 0){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mLength <= 0.0){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (wgrd < 0.0){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mArea <= 0.0){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (ratio < 0.0){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mUpperArea < 0.0){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }

  for (int i = 0; i < mNumRps; i++) {
    mCqup[i] = ratio;
  }

/*
  ΥץǤ x ͤƤʤΤǻѤޤ
  mWidth = mArea / mLength;
  double angle;
  angle = atan(wgrd);
  mSin = sin(angle);

  //; mCos 롣mCos  x ˤȤä˹߿嶯
  //; ˾褸ޤ
  mCos = cos(angle);
*/

  int dct;
  if (fscanf(fp, "%lf %d %d %lf",
    &mAlpha, &mM, &dct, &mCprec) != 4) {
    Eprint(__FILE__, __LINE__);
    Oprint();
    exit(1);
  }
  mRpClm.Set_data_connection_type(dct);
  for(int i = 0; i < mNumRps; i++) {
    if (fscanf(fp, "%d", &dct) != 1) {
      Eprint(__FILE__, __LINE__);
      Oprint();
      exit(1);
    }
    mpRpSlope[i].Set_data_connection_type(dct);
  }

  if (fscanf(fp, "%lf %lf", &mSlevel, &mEps) != 2) {
    Eprint(__FILE__, __LINE__);
    Oprint();
    exit(1);
  }
  mSsmall = mSlevel * mEps;

  fscanf(fp, "%d", &mFlag);  // monitor flag
  if (mFlag == 1) {
    char mfilename[128];
    mfilename[0] = '\0';
    char absname[128];
    Object::Absname(absname);
    strcat(mfilename, "monitor_storage");
    strcat(mfilename, absname);
    khmkdir(mfilename, 493); // create the directory mfilename
                             // with the mode 493 = 0755
    strcat(mfilename, "/monitor_storage.dat");
    monitor_storage.open(mfilename, ios::out);
    monitor_storage << "# monitor of " << absname << endl;
    mfilename[0] = '\0';

    strcat(mfilename, "monitor_deltaT");
    strcat(mfilename, absname);
    khmkdir(mfilename, 493); // create the directory mfilename
                             // with the mode 493 = 0755
    strcat(mfilename, "/monitor_deltaT.dat");
    monitor_deltaT.open(mfilename, ios::out);
    monitor_deltaT << "# monitor of " << absname << endl;
    mfilename[0] = '\0';
   }
}

//;  ̤
void
TramStorageSlope::Set_initial_state(FILE* fp)
{
  if (fscanf(fp, "%ld %lf %lf %ld", &current_time,
    &mS_crnt, &mrPsum_last, &mDt_next) != 4) {
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mS_crnt < 0.){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mrPsum_last < 0.){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }
  if (mDt_next < 0.){
    Eprint(__FILE__,__LINE__);Oprint();exit(1);
  }

  mHalvedp = YES;
  mDoubledp = NO;
  mTime = 0;
  mTime_max = 5;

} // Set_initial_state

//;  Work
//; Can_you_calculate  Calculate ꤷƲ
Boolean
TramStorageSlope::Work(void)
{

//  if (current_time >= MediumTermTargetTime())
//  { return NO; }        //; ׻Ѥ

// 000904mk
  // <japanese>
  // ׻׵ʤcurrenet_time ɸۤƤз׻
  // 
  // </japanese>
  if (mCreqp == NO && current_time >= MediumTermTargetTime()) return NO;
// 000904mk

  //; ٤Ǥ׻¹Ԥ calc = YES Ȥʤ
  Boolean calc = NO;

 LOOP:	//; 롼 "LOOP" 

  //; ƥåפη׻.
  if (Calculate() == NO) {
    if (calc == NO) return NO;	//; ׻˽λ
    goto FINISH;
  }

  calc = YES;
  mCreqp = NO; // reset mCreqp

  //; ߻ (׻Ѥ߻) current_time ι.
  if (time_step <= 0) goto FINISH;
  current_time += time_step;
  if (current_time >= MediumTermTargetTime()) goto FINISH;

  goto LOOP;	//; ƥåפη׻.

  //; 롼 "LOOP" ޤ.

 FINISH:	//; ׻ν.

  Set_data_pack_array();	//; ǡѥåκ.
  Reduce_data_pack_stock();	//; ץǡѥåξõ.

  return YES;
}

//;  
Boolean
TramStorageSlope::Initial_output(void)
{
  mQ_crnt = mAlpha * pow(mS_crnt, mM);
  mSpSlope.Set_comment_string("unix time (sec), discharge (m^3/sec)");
  double discharge = mQ_crnt*mArea;
  mSpSlope.Send_data(current_time, discharge);

  return YES;
}

//;  ǽ֤ե¸
void
TramStorageSlope::Save_terminal_state(FILE* fp)
{
  if (fp != NULL) {
    fprintf(fp, "%ld %ld %f %f %ld", mNum, current_time, mS_crnt,
      mrPsum_last, mDt_next);
  }
} // Save_terminal_state

//time_t
//TramStorageSlope::MediumTermTargetTime(void) {
//  return ((TramStorageSubBasin*)owner)->MediumTermTargetTime();
//}

//;  ꥭ졼
Boolean
TramStorageSlope::Calculate()
{
/*
 storage ˤĤ
   ü
     q_i = \alpha s_i$^m$
     frac{d s_i}{dt}
     = r \Sigma^{M_i}_{j=i}Q_i/A_i - \alpha s_i$^m$
     Q_i = q_i A_i
    tram ˡѤƲ
*/

  //; time_step : Υॹƥå
  //; mDt_next : δΥॹƥå
  //; dta : ॹƥåפκѿ
  time_step = Calculate_time_step();
//  time_step = mDt_next;
  if (Can_you_calculate() == NO) { return NO; }

  long dta = time_step;

  if (mFlag == 1) {
    monitor_storage << current_time;
    monitor_deltaT << current_time;
  }

  double s_next = 0.;
 FIRST:

  //; ή (m)
  //; ߱
  double rPsum = mCprec *
  mRpClm.Get_mean_data(current_time,current_time+dta) * dta;

  //; ή¦ή̤ѤñѤ˴
  for (int i = 0; i < mNumRps; i++) {
    rPsum += mCqup[i] *
    mpRpSlope[i].Get_mean_data(current_time,current_time+dta)
    * dta / mArea;
  }

  mQ_crnt = mAlpha * pow(mS_crnt, mM);
  if (mHalvedp == YES) {     //; ׻ϻ⤷ϡ׻ֳִ֤
			     //; Ⱦʬˤʤäľ
    //; AB-1 Eulerˡˤͽ¬
    s_next = mS_crnt - dta * mQ_crnt + rPsum;
    //; 礭ưä硢ʲξ֤ʤ dta ȾʬˤƺƷ
    //; 롣
    if ((mS_crnt > mSlevel * mEps * 10.)
      &&(s_next < 0.1 * mS_crnt)) {
      if (dta > 1) {
	dta = dta/2;
	mDt_next = dta;
	goto FIRST;
      }
    }
    if (s_next < 0.) s_next = 0.;

    while (1) {
      mQ_next = mAlpha * pow(s_next, mM);

      //; AB-2 Ūˤ뽤
      mS_next = mS_crnt + dta * (- mQ_next - mQ_crnt)/2. + rPsum;
      //; 礭ưä硢ʲξ֤ʤ dta Ⱦʬˤƺ
      //; ׻롣
      if ((mS_crnt > mSlevel * mEps * 10.)
	&&(mS_next < 0.1 * mS_crnt)) {
	if (dta > 1) {
	  dta = dta/2;
	  mDt_next = dta;
	  goto FIRST;
	}
      }
      if (mS_next < 0.) mS_next = 0.;
      if (fabs(mS_next - s_next) < mSsmall) { break; }
      s_next = mS_next;
    }
    mHalvedp = NO;
  }
  else {                      //; ̾ξ
    //; AB-1 ˤͽ¬
    s_next = mS_last - 2. * dta * mQ_crnt + rPsum + mrPsum_last;
    //; 礭ưä硢ʲξ֤ʤ dta Ⱦʬˤƺ
    //; ׻롣
    if ((mS_crnt > mSlevel * mEps * 10.)
      &&(mS_next < 0.1 * mS_crnt)) {
      if (dta > 1) {
	dta = dta/2;
	mDt_next = dta;
	mHalvedp = YES;
	goto FIRST;
      }
    }
    if (s_next < 0.) s_next = 0.;
    mQ_next = mAlpha * pow(s_next, mM);

    //; AB-2 Ūˤ뽤
    mS_next = mS_crnt + dta * (- mQ_next - mQ_crnt)/2. + rPsum;

    //; 礭ưä硢ʲξ֤ʤ dta Ⱦʬˤƺ
    //; ׻롣
    if ((mS_crnt > mSlevel * mEps * 10.)
      &&(mS_next < 0.1 * mS_crnt)) {
      if (dta > 1) {
	dta = dta/2;
	mDt_next = dta;
	mHalvedp = YES;
	goto FIRST;
      }
    }
    if (mS_next < 0.) mS_next = 0.;

    //; ɾ
    if (fabs(mS_next - s_next)/6. < mSsmall) {
      mTime++;
      if (mTime == mTime_max) {
	//; ׻ֳִ֤ܤˤ
	if (dta < MaxDT) {
	  mDt_next = dta * 2;
	  mTime = 0;
	  mDoubledp = YES;
	}
      }
    }
    else {
      if (dta > 1) {
	//; ׻ֳִ֤Ⱦʬˤ
	dta = dta/2;
	mDt_next = dta;
	mTime = 0;
	mHalvedp = YES;
	goto FIRST;
      }
    }
  }

  //; 
  mQ_next = mAlpha * pow(mS_next, mM);
  double discharge_next = mQ_next * mArea;
  time_step = dta;
  mSpSlope.Send_data(current_time+time_step, discharge_next);

  if (mFlag == 1) {
    monitor_storage << " " << mS_crnt << endl;
    monitor_deltaT << " " << time_step << endl;
  }

//  cout << current_time+time_step << " " << time_step << " "
//       << discharge_next << " " << mTime << endl;

  //; ѿι
  if (mDoubledp == YES) {
    mrPsum_last += rPsum;
    mDoubledp = NO;
  } else {
    mS_last = mS_crnt;
    mrPsum_last = rPsum;
  }
  mS_crnt = mS_next;

  return YES;
}

/*; end of file */
