/*;  scftotal.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl scftotal.h\quad\thepage}} */
/*;  $$Date: 2000/12/02 09:58:53 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/scftotal.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) scftotal.h -- total system created from a structure configuration file

/*
{\ {\setlength{\parskip}{4pt}% latex-mode
...
}% end of latex-mode }
*/

#if !defined(scftotal_h)
#define scftotal_h

#include "total.h"

/*----------------------------------------------------------------------
CLASS
    ScfTotalSystem

    total system class confugured from a "structure configuration file"

DESCRIPTION
    total system class configured from a structure configuration file

    <japanese>
    ScfTotalSystem ϡüҤ乽Ǥιե Structure
    Configuration File ɤߤȤäƺꤢηϤΥ饹Ǥ

    </japanese>

----------------------------------------------------------------------*/

class ScfTotalSystem : public Total_system {
protected:

  //////////////////////////////////////////////////////////////////////
  // Pointer to the area where pointers to relay ports for receiveing
  // data are stored
  Port** mRps;
  //////////////////////////////////////////////////////////////////////
  // Number of relay ports for receiving data
  int mNumRps;

  //////////////////////////////////////////////////////////////////////
  // Pointer to the area where pointers to relay ports for sending
  // data are stored
  Port** mSps;
  //////////////////////////////////////////////////////////////////////
  // Number of relay ports for sending data
  int mNumSps;

  //////////////////////////////////////////////////////////////////////
  // Usual (Non-iterate) Components
  Element** mElmnt;

  //////////////////////////////////////////////////////////////////////
  // Number of usual components
  int mNumElmnt;

  //////////////////////////////////////////////////////////////////////
  // Iterator Components
  Element** mIte;

  //////////////////////////////////////////////////////////////////////
  // Number of iterator components
  int mNumIte;

  //////////////////////////////////////////////////////////////////////
  // Structure Configuration File
  char* mScffile;

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ¦üҤϿޤ饹ޤConstruct 
  // ƤФޤϿƤʤ¦üҤ³Ǥޤ.
  // </japanese>
  void Register_input_ports(void);
  void Register_receive_ports(void) { Register_output_ports(); }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ¦üҤϿޤ饹ޤConstruct 
  // ƤФޤϿƤʤ¦üҤ³Ǥޤ.
  // </japanese>
  void Register_output_ports(void);
  void Register_send_ports(void) { Register_input_ports(); }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ǤΤ, ȿ׻Ԥʤʤ֥ȤΥɥ쥹
  // component_array Ͽޤ.
  //
  // ϿƤʤ֥Ȥˤ, ׻¹̿᤬ޤ.
  //
  // 㤨, 饹ǹ֥ abc ǡФȤ
  // , Τ褦 abc Ͽޤ.
  // <pre>
  //   void Register_components(void) { Register((void*) &abc); }
  // </pre>
  // λ, ɬ &abc  void* ˥㥹ȤƲ.
  //
  // ˤ &abc , Ū object_list ˼Ͽ줿, ǽŪ
  // void*  component_array ˵ޤ.
  // (Sub_system::Construct )
  //
  // </japanese>
  virtual void Register_components(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Register_iterators , ǤΤ, ȿ׻ԤʤǤΥ
  // 쥹 iterator_array Ͽޤ. ȿ׻Ԥʤʬϥǥ
  // ˺ץޤδؿʤФʤޤ
  //
  // ScfTotalSystem Ѥ Register_iterators ϡScfTotal.cc ǼƤޤ
  // </japanese>
  void Register_iterators(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ʬϥǥüҤ³ Sub_system::Connect Ѥ
  // ޤ.
  //
  // ScfTotalSystem Ѥ Connect_ports ScfTotal.cc ǼƤޤ
  // </japanese>
  void Connect_ports(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ʬϥǥľ̿³ Element::Register(void*,int) 
  // Ѥޤ.
  //
  // ScfTotalSystem Ѥ Connect_components ScfTotal.cc ǼƤޤ
  // </japanese>
  void Connect_components(void);

public:
  //////////////////////////////////////////////////////////////////////
  // Set initial state by reading a file
  virtual void Set_initial_state(FILE* fp) {
    fscanf(fp, "%ld", &current_time);
    ignore(fp, '\n');
  }

  //////////////////////////////////////////////////////////////////////
  // Default Constructor
  //
  //  class_name = "ScfTotalSystem";
  ScfTotalSystem(char* filename);

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~ScfTotalSystem() { delete [] mRps; delete [] mSps; delete [] mElmnt;
  delete [] mIte; delete [] mScffile; }

};

#endif // scftotal_h

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
